@@ -1,5 +1,7 @@
 // -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 // vi: set et ts=4 sw=2 sts=2:
+// SPDX-FileCopyrightInfo: Copyright (C) DUNE Project contributors, see file LICENSE.md in module root
+// SPDX-License-Identifier: LicenseRef-GPL-2.0-only-with-DUNE-exception
 #ifndef DUNE_GRID_COMMON_REFINEMENT_SIMPLEX_CC
 #define DUNE_GRID_COMMON_REFINEMENT_SIMPLEX_CC
 
@@ -104,7 +106,7 @@
    \f$(x_0,x_1,x_2)=(2,2,2)\f$.
    - First we count the number of vertices in the green tetrahedron of
     width \f$x_0-1=1\f$ (4).  Then we take the green tetrahedron away.
-    Whats left is a triangle, which extends into the (1,2)-plane.
+    What's left is a triangle, which extends into the (1,2)-plane.
    - Now we count the number of vertices in the red triangle of width
     \f$x_1-1=1\f$ (3).  Again we take the counted points away and are
     left with a line which extends into direction 2.
@@ -246,11 +248,14 @@
     - \f$x_{p_d}:=x_{p_d}-x_{p_{d+1}}\f$.
  */
 
+#include <algorithm>
+
 #include <dune/common/fvector.hh>
+#include <dune/common/math.hh>
 
 #include <dune/geometry/multilineargeometry.hh>
-#include <dune/geometry/genericgeometry/topologytypes.hh>
 #include <dune/geometry/referenceelements.hh>
+#include <dune/geometry/type.hh>
 
 #include "base.cc"
 
@@ -276,7 +281,11 @@ namespace Dune {
       /*! @brief Calculate n!
 
          Runtime is of order O(n).
+
+         \deprecate Use factorial from dune/common/math instead. Will be
+                    removed after Dune 2.9.
        */
+      [[deprecated("Use factorial from dune-common's math.hh")]]
       inline int factorial(int n)
       {
         int prod = 1;
@@ -288,17 +297,20 @@ namespace Dune {
       /*! @brief calculate \f$\left({upper}\atop{lower}\right)\f$
 
          Runtime is of order O(min {lower, upper-lower})
+
+         \deprecate Use binomial from dune/common/math instead. Will be
+                    removed after Dune 2.9.
        */
+      [[deprecated("Use binomial from dune-common's math.hh")]]
       inline int binomial(int upper, int lower)
       {
-        if(lower > upper - lower)
-          lower = upper - lower;
+        lower = std::min( lower, upper - lower );
         if(lower < 0)
           return 0;
         int prod = 1;
-        for(int i = upper - lower + 1; i <= upper; ++i)
-          prod *= i;
-        return prod / factorial(lower);
+        for(int i = upper - lower; i < upper; ++i)
+          prod *= (i+1);
+        return prod / Dune::factorial(lower);
       }
 
       /*! @brief calculate the index of a given gridpoint within a
@@ -312,7 +324,7 @@ namespace Dune {
       {
         int index = 0;
         for(int i = 0; i < dimension; ++i)
-          index += binomial(dimension-i + point[i]-1, dimension-i);
+          index += Dune::binomial(dimension-i + point[i]-1, dimension-i);
         return index;
       }
 
@@ -420,9 +432,8 @@ namespace Dune {
       class RefinementImp
       {
       public:
-        enum { dimension = dimension_ };
+        constexpr static int dimension = dimension_;
         typedef CoordType ctype;
-        enum { dimensionworld = dimension };
 
         template<int codimension>
         struct Codim;
@@ -431,13 +442,13 @@ namespace Dune {
         typedef typename Codim<0>::SubEntityIterator ElementIterator;
         typedef FieldVector<int, dimension+1> IndexVector;
 
-        static int nVertices(int level);
-        static VertexIterator vBegin(int level);
-        static VertexIterator vEnd(int level);
+        static int nVertices(int nIntervals);
+        static VertexIterator vBegin(int nIntervals);
+        static VertexIterator vEnd(int nIntervals);
 
-        static int nElements(int level);
-        static ElementIterator eBegin(int level);
-        static ElementIterator eEnd(int level);
+        static int nElements(int nIntervals);
+        static ElementIterator eBegin(int nIntervals);
+        static ElementIterator eEnd(int nIntervals);
       };
 
       template<int dimension, class CoordType>
@@ -452,49 +463,49 @@ namespace Dune {
       template<int dimension, class CoordType>
       int
       RefinementImp<dimension, CoordType>::
-      nVertices(int level)
+      nVertices(int nIntervals)
       {
-        return binomial(dimension + (1 << level), dimension);
+        return Dune::binomial(dimension + nIntervals, (int)dimension);
       }
 
       template<int dimension, class CoordType>
       typename RefinementImp<dimension, CoordType>::VertexIterator
       RefinementImp<dimension, CoordType>::
-      vBegin(int level)
+      vBegin(int nIntervals)
       {
-        return VertexIterator(level);
+        return VertexIterator(nIntervals);
       }
 
       template<int dimension, class CoordType>
       typename RefinementImp<dimension, CoordType>::VertexIterator
       RefinementImp<dimension, CoordType>::
-      vEnd(int level)
+      vEnd(int nIntervals)
       {
-        return VertexIterator(level, true);
+        return VertexIterator(nIntervals, true);
       }
 
       template<int dimension, class CoordType>
       int
       RefinementImp<dimension, CoordType>::
-      nElements(int level)
+      nElements(int nIntervals)
       {
-        return 1 << (level * dimension);
+        return Dune::power(nIntervals, int(dimension));
       }
 
       template<int dimension, class CoordType>
       typename RefinementImp<dimension, CoordType>::ElementIterator
       RefinementImp<dimension, CoordType>::
-      eBegin(int level)
+      eBegin(int nIntervals)
       {
-        return ElementIterator(level);
+        return ElementIterator(nIntervals);
       }
 
       template<int dimension, class CoordType>
       typename RefinementImp<dimension, CoordType>::ElementIterator
       RefinementImp<dimension, CoordType>::
-      eEnd(int level)
+      eEnd(int nIntervals)
       {
-        return ElementIterator(level, true);
+        return ElementIterator(nIntervals, true);
       }
 
       // //////////////
@@ -516,7 +527,7 @@ namespace Dune {
         typedef typename Refinement::template Codim<dimension>::Geometry Geometry;
         typedef RefinementIteratorSpecial<dimension, CoordType, dimension> This;
 
-        RefinementIteratorSpecial(int level, bool end = false);
+        RefinementIteratorSpecial(int nIntervals, bool end = false);
 
         void increment();
         bool equals(const This &other) const;
@@ -534,8 +545,8 @@ namespace Dune {
 
       template<int dimension, class CoordType>
       RefinementIteratorSpecial<dimension, CoordType, dimension>::
-      RefinementIteratorSpecial(int level, bool end)
-        : size(1<<level)
+      RefinementIteratorSpecial(int nIntervals, bool end)
+        : size(nIntervals)
       {
         vertex[0] = (end) ? size + 1 : 0;
         for(int i = 1; i < dimension; ++ i)
@@ -584,7 +595,7 @@ namespace Dune {
       {
         std::vector<CoordVector> corners(1);
         corners[0] = (CoordVector)vertex;
-        return Geometry(GeometryType(0), corners);
+        return Geometry(GeometryTypes::vertex, corners);
       }
 
       template<int dimension, class CoordType>
@@ -607,7 +618,7 @@ namespace Dune {
         typedef typename Refinement::template Codim<0>::Geometry Geometry;
         typedef RefinementIteratorSpecial<dimension, CoordType, 0> This;
 
-        RefinementIteratorSpecial(int level, bool end = false);
+        RefinementIteratorSpecial(int nIntervals, bool end = false);
 
         void increment();
         bool equals(const This &other) const;
@@ -623,7 +634,7 @@ namespace Dune {
 
       protected:
         typedef FieldVector<int, dimension> Vertex;
-        enum { nKuhnSimplices = Factorial<dimension>::factorial };
+        constexpr static int nKuhnIntervals = Dune::factorial(dimension);
 
         Vertex origin;
         int kuhnIndex;
@@ -633,13 +644,13 @@ namespace Dune {
 
       template<int dimension, class CoordType>
       RefinementIteratorSpecial<dimension, CoordType, 0>::
-      RefinementIteratorSpecial(int level, bool end)
-        : kuhnIndex(0), size(1<<level), index_(0)
+      RefinementIteratorSpecial(int nIntervals, bool end)
+        : kuhnIndex(0), size(nIntervals), index_(0)
       {
         for(int i = 0; i < dimension; ++i)
           origin[i] = 0;
         if(end) {
-          index_ = Refinement::nElements(level);
+          index_ = Refinement::nElements(nIntervals);
           origin[0] = size;
         }
       }
@@ -655,7 +666,7 @@ namespace Dune {
 
         while(1) {
           ++kuhnIndex;
-          if(kuhnIndex == nKuhnSimplices) {
+          if(kuhnIndex == nKuhnIntervals) {
             kuhnIndex = 0;
             // increment origin
             for(int i = dimension - 1; i >= 0; --i) {
@@ -737,9 +748,7 @@ namespace Dune {
       RefinementIteratorSpecial<dimension, CoordType, 0>::geometry () const
       {
         std::vector<CoordVector> corners(dimension+1);
-        CoordVector v;
-        const ReferenceElement<CoordType, dimension> &refelem =
-          ReferenceElements<CoordType, dimension>::simplex();
+        auto refelem = ReferenceElements<CoordType, dimension>::simplex();
         for(int i = 0; i <= dimension; ++i)
           corners[i] = global(refelem.position(i, dimension));
         return Geometry(refelem.type(), corners);
@@ -767,7 +776,7 @@ namespace Dune {
       public:
         typedef RefinementImp<dimension, CoordType> Refinement;
 
-        SubEntityIterator(int level, bool end = false);
+        SubEntityIterator(int nIntervals, bool end = false);
       };
 
 #ifndef DOXYGEN
@@ -775,8 +784,8 @@ namespace Dune {
       template<int dimension, class CoordType>
       template<int codimension>
       RefinementImp<dimension, CoordType>::Codim<codimension>::SubEntityIterator::
-      SubEntityIterator(int level, bool end)
-        : RefinementIteratorSpecial<dimension, CoordType, codimension>(level, end)
+      SubEntityIterator(int nIntervals, bool end)
+        : RefinementIteratorSpecial<dimension, CoordType, codimension>(nIntervals, end)
       {}
 
 #endif
@@ -798,10 +807,10 @@ namespace Dune {
         int dim>
     struct Traits<
         topologyId, CoordType, coerceToId, dim,
-        typename enable_if<
-            ((GenericGeometry::SimplexTopology<dim>::type::id >> 1) ==
+        typename std::enable_if<
+            ((GeometryTypes::simplex(dim).id() >> 1) ==
              (topologyId >> 1) &&
-             (GenericGeometry::SimplexTopology<dim>::type::id >> 1) ==
+             (GeometryTypes::simplex(dim).id() >> 1) ==
              (coerceToId >> 1)
             )>::type
         >
