@@ -1,18 +1,36 @@
 // -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 // vi: set et ts=4 sw=2 sts=2:
+// SPDX-FileCopyrightInfo: Copyright (C) DUNE Project contributors, see file LICENSE.md in module root
+// SPDX-License-Identifier: LicenseRef-GPL-2.0-only-with-DUNE-exception
 
 #include <algorithm>
 #include <limits>
 #include <iostream>
+#include <type_traits>
 
 #include <config.h>
 
+#include <dune/common/math.hh>
+#include <dune/common/quadmath.hh>
 #include <dune/geometry/referenceelements.hh>
 #include <dune/geometry/quadraturerules.hh>
 #include <dune/geometry/quadraturerules/compositequadraturerule.hh>
+#include <dune/geometry/refinement.hh>
 
 bool success = true;
 
+template <class ctype, std::enable_if_t<std::numeric_limits<ctype>::is_specialized, int> = 0>
+ctype eps ()
+{
+  return std::numeric_limits<ctype>::epsilon();
+}
+
+template <class ctype, std::enable_if_t<!std::numeric_limits<ctype>::is_specialized, int> = 0>
+ctype eps ()
+{
+  return std::numeric_limits<float>::epsilon();
+}
+
 /*
    This is a simple accuracy test on the reference element. It integrates
    x^p and y^p with the quadrature rule of order p, which should give
@@ -31,7 +49,7 @@ ctype analyticalSolution (Dune::GeometryType t, int p, int direction )
 
   if (t.isCube())
   {
-    exact=1.0/(p+1);
+    exact = ctype( 1 )/(p+1);
     return exact;
   }
 
@@ -50,14 +68,14 @@ ctype analyticalSolution (Dune::GeometryType t, int p, int direction )
     const int pdim = (dim > 0 ? dim-1 : 0);
     if( direction < dim-1 )
     {
-      GeometryType nt( GeometryType::simplex, dim-1 );
+      Dune::GeometryType nt = Dune::GeometryTypes::simplex( dim-1 );
       if( dim > 0 )
         exact = analyticalSolution< ctype, pdim >( nt, p, direction );
       else
         exact = ctype( 1 );
     }
     else
-      exact = ctype( 1 ) / ctype( Dune::Factorial< pdim >::factorial * (p+1));
+      exact = ctype( 1 ) / ctype( Dune::factorial(pdim) * (p+1));
     return exact;
   }
 
@@ -67,10 +85,10 @@ ctype analyticalSolution (Dune::GeometryType t, int p, int direction )
     {
     case 0 :
     case 1 :
-      exact=1.0/((p+3)*(p+1));
+      exact = ctype( 1 )/((p+3)*(p+1));
       break;
     case 2 :
-      exact=2.0/((p+1)*(p+2)*(p+3));
+      exact = ctype( 2 )/((p+1)*(p+2)*(p+3));
       break;
     };
     return exact;
@@ -83,6 +101,7 @@ ctype analyticalSolution (Dune::GeometryType t, int p, int direction )
 template<class QuadratureRule>
 void checkQuadrature(const QuadratureRule &quad)
 {
+  using std::pow; using std::abs;
   using namespace Dune;
   typedef typename QuadratureRule::CoordType ctype;
   const unsigned int dim = QuadratureRule::d;
@@ -96,28 +115,26 @@ void checkQuadrature(const QuadratureRule &quad)
     const ctype weight = qp->weight();
 
     for (unsigned int d=0; d<dim; d++)
-      integral[d] += weight*std::pow(x[d],double(p));
+      integral[d] += weight*pow(x[d], p);
   }
 
   ctype maxRelativeError = 0;
   for(unsigned int d=0; d<dim; d++)
   {
     ctype exact = analyticalSolution<ctype,dim>(t,p,d);
-    ctype relativeError = std::abs(integral[d]-exact) /
-                          (std::abs(integral[d])+std::abs(exact));
+    ctype relativeError = abs(integral[d] - exact) / (abs(integral[d]) + abs(exact));
     if (relativeError > maxRelativeError)
       maxRelativeError = relativeError;
   }
-  ctype epsilon = std::pow(2.0,double(p))*p*std::numeric_limits<double>::epsilon();
+  ctype epsilon = pow(2.0,p)*p*eps<ctype>();
   if (p==0)
-    epsilon = 2.0*std::numeric_limits<double>::epsilon();
+    epsilon = 2.0*eps<ctype>();
   if (maxRelativeError > epsilon) {
     std::cerr << "Error: Quadrature for " << t << " and order=" << p << " failed" << std::endl;
     for (unsigned int d=0; d<dim; d++)
     {
       ctype exact = analyticalSolution<ctype,dim>(t,p,d);
-      ctype relativeError = std::abs(integral[d]-exact) /
-                            (std::abs(integral[d])+std::abs(exact));
+      ctype relativeError = abs(integral[d] - exact) / (abs(integral[d]) + abs(exact));
       std::cerr << "       relative error " << relativeError << " in direction " << d << " (exact = " << exact << " numerical = " << integral[d] << ")" << std::endl;
     }
     success = false;
@@ -127,21 +144,21 @@ void checkQuadrature(const QuadratureRule &quad)
 template<class QuadratureRule>
 void checkWeights(const QuadratureRule &quad)
 {
+  using std::abs;
   typedef typename QuadratureRule::CoordType ctype;
   const unsigned int dim = QuadratureRule::d;
   const unsigned int p = quad.order();
   const Dune::GeometryType& t = quad.type();
   typedef typename QuadratureRule::iterator QuadIterator;
-  double volume = 0;
+  ctype volume = 0;
   QuadIterator qp = quad.begin();
   QuadIterator qend = quad.end();
   for (; qp!=qend; ++qp)
   {
     volume += qp->weight();
   }
-  if (std::abs(volume -
-               Dune::ReferenceElements<ctype, dim>::general(t).volume())
-      > quad.size()*std::numeric_limits<double>::epsilon())
+  if (abs(volume - Dune::ReferenceElements<ctype, dim>::general(t).volume())
+      > quad.size()*eps<ctype>())
   {
     std::cerr << "Error: Quadrature for " << t << " and order=" << p
               << " does not sum to volume of RefElem" << std::endl;
@@ -155,18 +172,28 @@ void checkWeights(const QuadratureRule &quad)
 }
 
 template<class ctype, int dim>
-void check(const Dune::GeometryType::BasicType &btype,
+void check(Dune::GeometryType type,
            unsigned int maxOrder,
            Dune::QuadratureType::Enum qt = Dune::QuadratureType::GaussLegendre)
 {
   typedef Dune::QuadratureRule<ctype, dim> Quad;
-  Dune::GeometryType t(btype,dim);
-
+  std::string qt_str = "";
+  switch (qt) {
+    case Dune::QuadratureType::GaussLegendre: qt_str = "GaussLegendre"; break;
+    case Dune::QuadratureType::GaussJacobi_1_0: qt_str = "GaussJacobi_1_0"; break;
+    case Dune::QuadratureType::GaussJacobi_2_0: qt_str = "GaussJacobi_2_0"; break;
+    case Dune::QuadratureType::GaussJacobi_n_0: qt_str = "GaussJacobi_n_0"; break;
+    case Dune::QuadratureType::GaussLobatto: qt_str = "GaussLobatto"; break;
+    case Dune::QuadratureType::GaussRadauLeft: qt_str = "GaussRadauLeft"; break;
+    case Dune::QuadratureType::GaussRadauRight: qt_str = "GaussRadauRight"; break;
+    default: qt_str = "unknown";
+  }
+  std::cout << "check(Quadrature of type " << qt_str << ")" << std::endl;
   for (unsigned int p=0; p<=maxOrder; ++p)
   {
-    const Quad & quad = Dune::QuadratureRules<ctype,dim>::rule(t, p, qt);
-    if (quad.type() != t || unsigned(quad.order()) < p) {
-      std::cerr << "Error: Type mismatch! Requested Quadrature for " << t
+    const Quad & quad = Dune::QuadratureRules<ctype,dim>::rule(type, p, qt);
+    if (quad.type() != type || unsigned(quad.order()) < p) {
+      std::cerr << "Error: Type mismatch! Requested Quadrature for " << type
                 << " and order=" << p << "." << std::endl
                 << "\tGot Quadrature for " << quad.type() << " and order="
                 << quad.order() << std::endl;
@@ -176,16 +203,15 @@ void check(const Dune::GeometryType::BasicType &btype,
     checkWeights(quad);
     checkQuadrature(quad);
   }
-  if (dim>0 && (dim>3 ||
-                btype==Dune::GeometryType::cube ||
-                btype==Dune::GeometryType::simplex))
+  if (dim>0 && (dim>3 || type.isCube() || type.isSimplex()))
   {
-    check<ctype,((dim==0) ? 0 : dim-1)>(btype, maxOrder, qt);
+    type = type.isCube() ? Dune::GeometryTypes::cube(dim-1) : Dune::GeometryTypes::simplex(dim-1);
+    check<ctype,((dim==0) ? 0 : dim-1)>(type , maxOrder, qt);
   }
 }
 
 template<class ctype, int dim>
-void checkCompositeRule(const Dune::GeometryType::BasicType &btype,
+void checkCompositeRule(Dune::GeometryType type,
                         unsigned int maxOrder,
                         unsigned int maxRefinement,
                         Dune::QuadratureType::Enum qt = Dune::QuadratureType::GaussLegendre)
@@ -193,21 +219,18 @@ void checkCompositeRule(const Dune::GeometryType::BasicType &btype,
   typedef Dune::QuadratureRule<ctype, dim> BaseQuad;
   typedef Dune::CompositeQuadratureRule<ctype, dim> Quad;
 
-  Dune::GeometryType t(btype,dim);
-
   for (unsigned int p=0; p<=maxOrder; ++p)
   {
-    const BaseQuad& baseQuad = Dune::QuadratureRules<ctype,dim>::rule(t, p, qt);
-    Quad quad = Quad(baseQuad, maxRefinement);
+    const BaseQuad& baseQuad = Dune::QuadratureRules<ctype,dim>::rule(type, p, qt);
+    Quad quad = Quad(baseQuad, Dune::refinementLevels(maxRefinement));
 
     checkWeights(quad);
     checkQuadrature(quad);
   }
-  if (dim>0 && (dim>3 ||
-                btype==Dune::GeometryType::cube ||
-                btype==Dune::GeometryType::simplex))
+  if (dim>0 && (dim>3 || type.isCube() || type.isSimplex()))
   {
-    check<ctype,((dim==0) ? 0 : dim-1)>(btype, maxOrder, qt);
+    type = type.isCube() ? Dune::GeometryTypes::cube(dim-1) : Dune::GeometryTypes::simplex(dim-1);
+    check<ctype,((dim==0) ? 0 : dim-1)>(type, maxOrder, qt);
   }
 }
 
@@ -220,16 +243,39 @@ int main (int argc, char** argv)
     std::cout << "maxOrder = " << maxOrder << std::endl;
   }
   try {
-    check<double,4>(Dune::GeometryType::cube, maxOrder);
-    check<double,4>(Dune::GeometryType::cube, std::min(maxOrder, unsigned(31)),
+    check<double,4>(Dune::GeometryTypes::cube(4), maxOrder);
+    check<double,4>(Dune::GeometryTypes::cube(4), std::min(maxOrder, unsigned(31)),
                     Dune::QuadratureType::GaussLobatto);
-    check<double,4>(Dune::GeometryType::simplex, maxOrder);
-    check<double,3>(Dune::GeometryType::prism, maxOrder);
-    check<double,3>(Dune::GeometryType::pyramid, maxOrder);
+    check<double,4>(Dune::GeometryTypes::cube(4), std::min(maxOrder, unsigned(30)),
+                    Dune::QuadratureType::GaussRadauLeft);
+    check<double,4>(Dune::GeometryTypes::cube(4), std::min(maxOrder, unsigned(30)),
+                    Dune::QuadratureType::GaussRadauRight);
+    check<double,4>(Dune::GeometryTypes::simplex(4), maxOrder);
+
+#if HAVE_LAPACK
+    check<double,4>(Dune::GeometryTypes::simplex(4), maxOrder, Dune::QuadratureType::GaussJacobi_n_0);
+#else
+    std::cout << "Skip GaussJacobi_n_0 tests as LAPACK has not been found" << std::endl;
+#endif
+
+    check<double,3>(Dune::GeometryTypes::prism, maxOrder);
+    check<double,3>(Dune::GeometryTypes::pyramid, maxOrder);
 
     unsigned int maxRefinement = 4;
 
-    checkCompositeRule<double,2>(Dune::GeometryType::simplex, maxOrder, maxRefinement);
+    checkCompositeRule<double,2>(Dune::GeometryTypes::triangle, maxOrder, maxRefinement);
+
+#if HAVE_QUADMATH
+    check<Dune::Float128,4>(Dune::GeometryTypes::cube(4), maxOrder);
+    check<Dune::Float128,4>(Dune::GeometryTypes::cube(4), std::min(maxOrder, unsigned(31)),
+                    Dune::QuadratureType::GaussLobatto);
+    check<Dune::Float128,4>(Dune::GeometryTypes::cube(4), std::min(maxOrder, unsigned(30)),
+                    Dune::QuadratureType::GaussRadauLeft);
+    check<Dune::Float128,4>(Dune::GeometryTypes::cube(4), std::min(maxOrder, unsigned(30)),
+                    Dune::QuadratureType::GaussRadauRight);
+#else
+    std::cout << "Skip Float128 tests as Quadmath is not supported" << std::endl;
+#endif
   }
   catch( const Dune::Exception &e )
   {
