@@ -1,5 +1,7 @@
 // -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 // vi: set et ts=4 sw=2 sts=2:
+// SPDX-FileCopyrightInfo: Copyright (C) DUNE Project contributors, see file LICENSE.md in module root
+// SPDX-License-Identifier: LicenseRef-GPL-2.0-only-with-DUNE-exception
 /** \file
     \brief A unit test for the ReferenceElements
     \todo For several element types the subEntities() method is not tested yet!
@@ -16,6 +18,70 @@ using namespace Dune;
 #define test(a) if (! (a) ) { std::cerr << __FILE__ << ":" << __LINE__ << ": Test `" # a "' failed" << std::endl; errors++; }
 #define testcmp(a,b) if (! (a == b) ) { std::cerr << __FILE__ << ":" << __LINE__ << ": Test `" # a " == " # b "' failed (got " << a << ")" << std::endl; errors++; }
 
+
+
+template<class RE>
+int checkSubEntities(const RE& re)
+{
+  int errors = 0;
+
+  for (std::size_t codim = 0; codim <= RE::dimension; ++codim)
+  {
+    for (std::size_t i = 0; i < std::size_t(re.size(codim)); ++i)
+    {
+      for (std::size_t c = 0; c <= RE::dimension; ++c)
+      {
+        auto subEntities = re.subEntities(i, codim, c);
+        auto it = subEntities.begin();
+        auto end = subEntities.end();
+
+        // check size of subEntities range
+        testcmp(re.size(i, codim, c), end-it);
+        testcmp(std::size_t(re.size(i, codim, c)), subEntities.size());
+
+        // check if sub-subentity range is empty if
+        // sub-subentity codim c < codim of subentity
+        if (c<codim)
+          testcmp(re.size(i, codim, c), 0);
+
+        // check if sub-subentity range is singleton if
+        // sub-subentity codim c = codim of subentity
+        if (c==codim)
+          testcmp(re.size(i, codim, c), 1);
+
+        // check if sub-subentity range has multiple
+        // entries for sub-subentity codim c > codim of subentity
+        if (c>codim)
+          test(re.size(i, codim, c) > 1);
+
+        // check if subEntities() is conforming with subEntity()
+        for(std::size_t j = 0; j < subEntities.size(); ++j)
+        {
+          testcmp(*it, std::size_t(re.subEntity(i, codim, j, c)));
+          ++it;
+        }
+
+        // check is contains is true for all indices in the range
+        for(auto j : subEntities)
+          test(subEntities.contains(j));
+
+        // check if contains is consistent
+        std::vector<bool> containedSubEntities(re.size(c), false);
+        for(auto j : subEntities)
+          containedSubEntities[j] = true;
+        for(auto j: range(std::size_t(re.size(c))))
+          testcmp(containedSubEntities[j],subEntities.contains(j));
+
+        // after incrementing size times, the range should be exhausted
+        testcmp(it, end);
+      }
+    }
+  }
+  return errors;
+}
+
+
+
 int main () try
 {
   int errors = 0;
@@ -32,9 +98,18 @@ int main () try
   //   Test segment
   // //////////////////////////////////////////////////////////////////////////
 
-  type.makeLine();
+  type = GeometryTypes::line;
 
-  const ReferenceElement<double,1>& referenceLine = ReferenceElements<double, 1>::general(type);
+  {
+    // check default constructibility and comparison operators
+    ReferenceElements<double,1>::ReferenceElement r1, r2;
+    test(r1 == r2);
+    test(not (r1 != r2));
+    // check hash value
+    testcmp(hash_value(r1),0);
+  }
+
+  const Transitional::ReferenceElement<double,Dim<1>> referenceLine = ReferenceElements<double, 1>::general(type);
 
   // size(int c)
   testcmp(referenceLine.size(0),1);
@@ -53,17 +128,23 @@ int main () try
   test(referenceLine.type(0,1).isVertex());
   test(referenceLine.type(1,1).isVertex());
 
+  // test the 'volume' method
+  decltype(referenceLine)::Volume lineVolume = referenceLine.volume();
+  testcmp(lineVolume, 1);
+
   // test the 'geometry' method
-  const ReferenceElement<double,1>::Codim<0>::Geometry referenceLineMapping = referenceLine.geometry< 0 >( 0 );
+  const ReferenceElements<double,1>::ReferenceElement::Codim<0>::Geometry referenceLineMapping = referenceLine.geometry< 0 >( 0 );
   referenceLineMapping.corner(0);
 
+  errors += checkSubEntities(referenceLine);
+
   // //////////////////////////////////////////////////////////////////////////
   //   Test triangle
   // //////////////////////////////////////////////////////////////////////////
 
-  type.makeTriangle();
+  type = GeometryTypes::triangle;
 
-  const ReferenceElement<double,2>& referenceTriangle = ReferenceElements<double, 2>::general(type);
+  const auto referenceTriangle = referenceElement<double,2>(type);
 
   // size(int c)
   testcmp(referenceTriangle.size(0),1);
@@ -121,25 +202,26 @@ int main () try
   test(referenceTriangle.type(1,2).isVertex());
   test(referenceTriangle.type(2,2).isVertex());
 
+  // test the 'volume' method
+  decltype(referenceTriangle)::Volume triangleVolume = referenceTriangle.volume();
+  testcmp(triangleVolume, 0.5);
+
   // test the 'geometry' method
-  const ReferenceElement<double,2>::Codim<0>::Geometry referenceTriangleMapping = referenceTriangle.geometry< 0 >( 0 );
+  const Transitional::ReferenceElement<double,Dim<2>>::Codim<0>::Geometry referenceTriangleMapping = referenceTriangle.geometry< 0 >( 0 );
   referenceTriangleMapping.corner(0);
 
-  // test the checkInside methods
+  // test the checkInside method
   test(referenceTriangle.checkInside({0.3,0.3}));
-  for (int i=0; i<3; i++)
-  {
-    test(referenceTriangle.checkInside<1>({0.5}, i));
-    test(!referenceTriangle.checkInside<1>({-0.5}, i));
-  }
+
+  errors += checkSubEntities(referenceTriangle);
 
   // //////////////////////////////////////////////////////////////////////////
   //   Test quadrilateral
   // //////////////////////////////////////////////////////////////////////////
 
-  type.makeQuadrilateral();
+  type = GeometryTypes::quadrilateral;
 
-  const ReferenceElement<double,2>& referenceQuad = ReferenceElements<double, 2>::general(type);
+  const Transitional::ReferenceElement<double,Dim<2>> referenceQuad = referenceElement<double>(type,Dim<2>());
 
   // size(int c)
   testcmp(referenceQuad.size(0),1);
@@ -208,18 +290,23 @@ int main () try
   test(referenceQuad.type(2,2).isVertex());
   test(referenceQuad.type(3,2).isVertex());
 
+  errors += checkSubEntities(referenceQuad);
+
+  // test the 'volume' method
+  decltype(referenceQuad)::Volume quadVolume = referenceQuad.volume();
+  testcmp(quadVolume, 1);
 
   // test the 'geometry' method
-  const ReferenceElement<double,2>::Codim<0>::Geometry referenceQuadMapping = referenceQuad.geometry< 0 >( 0 );
+  const Transitional::ReferenceElement<double,Dim<2>>::Codim<0>::Geometry referenceQuadMapping = referenceQuad.geometry< 0 >( 0 );
   referenceQuadMapping.corner(0);
 
   // //////////////////////////////////////////////////////////////////////////
   //   Test tetrahedron
   // //////////////////////////////////////////////////////////////////////////
 
-  type.makeTetrahedron();
+  type = GeometryTypes::tetrahedron;
 
-  const ReferenceElement<double,3>& referenceTetra = ReferenceElements<double, 3>::general(type);
+  const Transitional::ReferenceElement<double,Dim<3>> referenceTetra = referenceElement(double(),type,Dim<3>());
 
   // size(int c)
   testcmp(referenceTetra.size(0),1);
@@ -262,34 +349,26 @@ int main () try
   for (int i=0; i<referenceTetra.size(3); i++)
     test(referenceTetra.type(i,3).isVertex());
 
+  // test the 'volume' method
+  decltype(referenceTetra)::Volume tetraVolume = referenceTetra.volume();
+  testcmp(tetraVolume, 1.0/6.0);
+
   // test the 'geometry' method
-  const ReferenceElement<double,3>::Codim<0>::Geometry referenceTetraMapping = referenceTetra.geometry< 0 >( 0 );
+  const decltype(referenceTetra)::Codim<0>::Geometry referenceTetraMapping = referenceTetra.geometry< 0 >( 0 );
   referenceTetraMapping.corner(0);
 
-  // test the checkInside methods
+  // test the checkInside method
   test(referenceTetra.checkInside({0.3,0.3,0.3}));
 
-  // checkInside for edges
-  for (int i=0; i<6; i++)
-  {
-    test(referenceTetra.checkInside<2>({0.5}, i));
-    test(!referenceTetra.checkInside<2>({-0.5}, i));
-  }
-
-  // checkInside for faces
-  for (int i=0; i<4; i++)
-  {
-    test(referenceTetra.checkInside<1>({0.3,0.3}, i));
-    test(!referenceTetra.checkInside<1>({1.0,1.0}, i));
-  }
+  errors += checkSubEntities(referenceTetra);
 
   // //////////////////////////////////////////////////////////////////////////
   //   Test pyramid
   // //////////////////////////////////////////////////////////////////////////
 
-  type.makePyramid();
+  type = GeometryTypes::pyramid;
 
-  const ReferenceElement<double,3>& referencePyramid = ReferenceElements<double, 3>::general(type);
+  const auto referencePyramid = ReferenceElements<double, 3>::general(type);
 
   // size(int c)
   testcmp(referencePyramid.size(0),1);
@@ -344,17 +423,23 @@ int main () try
   for (int i=0; i<referencePyramid.size(3); i++)
     test(referencePyramid.type(i,3).isVertex());
 
+  // test the 'volume' method
+  decltype(referencePyramid)::Volume pyramidVolume = referencePyramid.volume();
+  testcmp(pyramidVolume, 1.0/3.0);
+
   // test the 'geometry' method
-  const ReferenceElement<double,3>::Codim<0>::Geometry referencePyramidMapping = referencePyramid.geometry< 0 >( 0 );
+  const auto referencePyramidMapping = referencePyramid.geometry< 0 >( 0 );
   referencePyramidMapping.corner(0);
 
+  errors += checkSubEntities(referencePyramid);
+
   // //////////////////////////////////////////////////////////////////////////
   //   Test prism
   // //////////////////////////////////////////////////////////////////////////
 
-  type.makePrism();
+  type = GeometryTypes::prism;
 
-  const ReferenceElement<double,3>& referencePrism = ReferenceElements<double, 3>::general(type);
+  const auto referencePrism = referenceElement<double,3>(type);
 
   // size(int c)
   testcmp(referencePrism.size(0),1);
@@ -409,17 +494,23 @@ int main () try
   for (int i=0; i<referencePrism.size(3); i++)
     test(referencePrism.type(i,3).isVertex());
 
+  // test the 'volume' method
+  decltype(referencePrism)::Volume prismVolume = referencePrism.volume();
+  testcmp(prismVolume, 0.5);
+
   // test the 'geometry' method
-  const ReferenceElement<double,3>::Codim<0>::Geometry referencePrismMapping = referencePrism.geometry< 0 >( 0 );
+  const auto referencePrismMapping = referencePrism.geometry< 0 >( 0 );
   referencePrismMapping.corner(0);
 
+  errors += checkSubEntities(referencePrism);
+
   // //////////////////////////////////////////////////////////////////////////
   //   Test hexahedron
   // //////////////////////////////////////////////////////////////////////////
 
-  type.makeHexahedron();
+  type = GeometryTypes::hexahedron;
 
-  const ReferenceElement<double,3>& referenceHexa = ReferenceElements<double, 3>::general(type);
+  const auto referenceHexa = ReferenceElements<double, 3>::general(type);
 
   // size(int c)
   testcmp(referenceHexa.size(0),1);
@@ -461,10 +552,16 @@ int main () try
   for (int i=0; i<referenceHexa.size(3); i++)
     test(referenceHexa.type(i,3).isVertex());
 
+  // test the 'volume' method
+  decltype(referenceHexa)::Volume hexaVolume = referenceHexa.volume();
+  testcmp(hexaVolume, 1);
+
   // test the 'geometry' method
-  const ReferenceElement<double,3>::Codim<0>::Geometry referenceHexaMapping = referenceHexa.geometry< 0 >( 0 );
+  const Transitional::ReferenceElement<double,Dim<3>>::Codim<0>::Geometry referenceHexaMapping = referenceHexa.geometry< 0 >( 0 );
   referenceHexaMapping.corner(0);
 
+  errors += checkSubEntities(referenceHexa);
+
   return errors>0 ? 1 : 0;
 
 }
