@@ -1,5 +1,7 @@
 // -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 // vi: set et ts=4 sw=2 sts=2:
+// SPDX-FileCopyrightInfo: Copyright (C) DUNE Project contributors, see file LICENSE.md in module root
+// SPDX-License-Identifier: LicenseRef-GPL-2.0-only-with-DUNE-exception
 
 #ifndef DUNE_GEOMETRY_VIRTUALREFINEMENT_CC
 #define DUNE_GEOMETRY_VIRTUALREFINEMENT_CC
@@ -33,33 +35,33 @@ namespace Dune
   template<int dimension, class CoordType>
   typename VirtualRefinement<dimension, CoordType>::VertexIterator
   VirtualRefinement<dimension, CoordType>::
-  vBegin(int level) const
+  vBegin(Dune::RefinementIntervals tag) const
   {
-    return VertexIterator(vBeginBack(level));
+    return VertexIterator(vBeginBack(tag));
   }
 
   template<int dimension, class CoordType>
   typename VirtualRefinement<dimension, CoordType>::VertexIterator
   VirtualRefinement<dimension, CoordType>::
-  vEnd(int level) const
+  vEnd(Dune::RefinementIntervals tag) const
   {
-    return VertexIterator(vEndBack(level));
+    return VertexIterator(vEndBack(tag));
   }
 
   template<int dimension, class CoordType>
   typename VirtualRefinement<dimension, CoordType>::ElementIterator
   VirtualRefinement<dimension, CoordType>::
-  eBegin(int level) const
+  eBegin(Dune::RefinementIntervals tag) const
   {
-    return ElementIterator(eBeginBack(level));
+    return ElementIterator(eBeginBack(tag));
   }
 
   template<int dimension, class CoordType>
   typename VirtualRefinement<dimension, CoordType>::ElementIterator
   VirtualRefinement<dimension, CoordType>::
-  eEnd(int level) const
+  eEnd(Dune::RefinementIntervals tag) const
   {
-    return ElementIterator(eEndBack(level));
+    return ElementIterator(eEndBack(tag));
   }
 
   //
@@ -264,18 +266,18 @@ namespace Dune
     template<int codimension>
     class SubEntityIteratorBack;
 
-    int nVertices(int level) const;
-    int nElements(int level) const;
+    int nVertices(Dune::RefinementIntervals tag) const;
+
+    int nElements(Dune::RefinementIntervals tag) const;
 
     static VirtualRefinementImp<topologyId, CoordType, coerceToId, dimension> &instance();
   private:
     VirtualRefinementImp() {}
-    static VirtualRefinementImp<topologyId, CoordType, coerceToId, dimension> *instance_;
 
-    typename VirtualRefinement::VertexIteratorBack *vBeginBack(int level) const;
-    typename VirtualRefinement::VertexIteratorBack *vEndBack(int level) const;
-    typename VirtualRefinement::ElementIteratorBack *eBeginBack(int level) const;
-    typename VirtualRefinement::ElementIteratorBack *eEndBack(int level) const;
+    typename VirtualRefinement::VertexIteratorBack *vBeginBack(Dune::RefinementIntervals tag) const;
+    typename VirtualRefinement::VertexIteratorBack *vEndBack(Dune::RefinementIntervals tag) const;
+    typename VirtualRefinement::ElementIteratorBack *eBeginBack(Dune::RefinementIntervals tag) const;
+    typename VirtualRefinement::ElementIteratorBack *eEndBack(Dune::RefinementIntervals tag) const;
   };
 
   template<unsigned topologyId, class CoordType,
@@ -283,58 +285,60 @@ namespace Dune
   VirtualRefinementImp<topologyId, CoordType, coerceToId, dimension> &
   VirtualRefinementImp<topologyId, CoordType, coerceToId, dimension>::instance()
   {
-    if(instance_ == 0)
-      instance_ = new VirtualRefinementImp<topologyId, CoordType, coerceToId, dimension>;
-    return *instance_;
+    static VirtualRefinementImp instance_{};
+    return instance_;
   }
 
-  template<unsigned topologyId, class CoordType,
-      unsigned coerceToId, int dimension>
-  VirtualRefinementImp<topologyId, CoordType, coerceToId, dimension> *
-  VirtualRefinementImp<topologyId, CoordType, coerceToId, dimension>::instance_ = 0;
-
-  template<unsigned topologyId, class CoordType,
+    template<unsigned topologyId, class CoordType,
       unsigned coerceToId, int dimension>
   int VirtualRefinementImp<topologyId, CoordType, coerceToId, dimension>::
-  nVertices(int level) const
+  nVertices(Dune::RefinementIntervals tag) const
   {
-    return StaticRefinement::nVertices(level);
+    return StaticRefinement::nVertices(tag);
   }
 
   template<unsigned topologyId, class CoordType,
       unsigned coerceToId, int dimension>
   typename VirtualRefinementImp<topologyId, CoordType, coerceToId, dimension>::VirtualRefinement::VertexIteratorBack *
   VirtualRefinementImp<topologyId, CoordType, coerceToId, dimension>::
-  vBeginBack(int level) const
-  { return new SubEntityIteratorBack<dimension>(StaticRefinement::vBegin(level)); }
+  vBeginBack(Dune::RefinementIntervals tag) const
+  {
+    return new SubEntityIteratorBack<dimension>(StaticRefinement::vBegin(tag));
+  }
 
   template<unsigned topologyId, class CoordType,
       unsigned coerceToId, int dimension>
   typename VirtualRefinementImp<topologyId, CoordType, coerceToId, dimension>::VirtualRefinement::VertexIteratorBack *
   VirtualRefinementImp<topologyId, CoordType, coerceToId, dimension>::
-  vEndBack(int level) const
-  { return new SubEntityIteratorBack<dimension>(StaticRefinement::vEnd(level)); }
+  vEndBack(Dune::RefinementIntervals tag) const
+  {
+    return new SubEntityIteratorBack<dimension>(StaticRefinement::vEnd(tag));
+  }
 
   template<unsigned topologyId, class CoordType,
       unsigned coerceToId, int dimension>
-  int VirtualRefinementImp<topologyId, CoordType, coerceToId, dimension>::nElements(int level) const
+  int VirtualRefinementImp<topologyId, CoordType, coerceToId, dimension>::nElements(Dune::RefinementIntervals tag) const
   {
-    return StaticRefinement::nElements(level);
+    return StaticRefinement::nElements(tag);
   }
 
   template<unsigned topologyId, class CoordType,
       unsigned coerceToId, int dimension>
   typename VirtualRefinementImp<topologyId, CoordType, coerceToId, dimension>::VirtualRefinement::ElementIteratorBack *
   VirtualRefinementImp<topologyId, CoordType, coerceToId, dimension>::
-  eBeginBack(int level) const
-  { return new SubEntityIteratorBack<0>(StaticRefinement::eBegin(level)); }
+  eBeginBack(Dune::RefinementIntervals tag) const
+  {
+    return new SubEntityIteratorBack<0>(StaticRefinement::eBegin(tag));
+  }
 
   template<unsigned topologyId, class CoordType,
       unsigned coerceToId, int dimension>
   typename VirtualRefinementImp<topologyId, CoordType, coerceToId, dimension>::VirtualRefinement::ElementIteratorBack *
   VirtualRefinementImp<topologyId, CoordType, coerceToId, dimension>::
-  eEndBack(int level) const
-  { return new SubEntityIteratorBack<0>(StaticRefinement::eEnd(level)); }
+  eEndBack(Dune::RefinementIntervals tag) const
+  {
+    return new SubEntityIteratorBack<0>(StaticRefinement::eEnd(tag));
+  }
 
   //
   // The iterator backend implementation
@@ -446,7 +450,7 @@ namespace Dune
     try {
       return backend == dynamic_cast<const This &>(other).backend;
     }
-    catch(std::bad_cast)
+    catch(const std::bad_cast&)
     {
       return false;
     }
@@ -520,8 +524,8 @@ namespace Dune
       topologyId &= ~1;
       coerceToId &= ~1;
 
-      const unsigned idSimplex = GenericGeometry::SimplexTopology<dimension>::type::id & ~1;
-      const unsigned idCube = GenericGeometry::CubeTopology<dimension>::type::id & ~1;
+      constexpr unsigned idSimplex = GeometryTypes::simplex(dimension).id() & ~1;
+      constexpr unsigned idCube = GeometryTypes::cube(dimension).id() & ~1;
 
       switch( topologyId )
       {
@@ -571,7 +575,7 @@ namespace Dune
       topologyId &= ~1;
       coerceToId &= ~1;
 
-      const unsigned idSimplex = GenericGeometry::SimplexTopology<dimension>::type::id & ~1;
+      constexpr unsigned idSimplex = GeometryTypes::simplex(dimension).id() & ~1;
 
       if (topologyId == 0 && coerceToId == 0)
         return VirtualRefinementImp< idSimplex, CoordType, idSimplex, dimension>::instance();
@@ -593,10 +597,10 @@ namespace Dune
       topologyId &= ~1;
       coerceToId &= ~1;
 
-      const unsigned idSimplex = GenericGeometry::SimplexTopology<dimension>::type::id & ~1;
-      const unsigned idCube = GenericGeometry::CubeTopology<dimension>::type::id & ~1;
-      const unsigned idPrism = GenericGeometry::PrismTopology<dimension>::type::id & ~1;
-      const unsigned idPyramid = GenericGeometry::PyramidTopology<dimension>::type::id & ~1;
+      constexpr unsigned idSimplex = GeometryTypes::simplex(dimension).id() & ~1;
+      constexpr unsigned idCube = GeometryTypes::cube(dimension).id() & ~1;
+      constexpr unsigned idPrism = GeometryTypes::prism.id() & ~1;
+      constexpr unsigned idPyramid = GeometryTypes::pyramid.id() & ~1;
 
       switch( topologyId )
       {
